@@ -2,6 +2,7 @@
 from __future__ import annotations
 
 import logging
+import typing
 from gettext import gettext as _
 
 import validators
@@ -14,10 +15,18 @@ from gsecrets.widgets.add_attribute_dialog import AddAttributeDialog
 from gsecrets.widgets.attachment_entry_row import AttachmentEntryRow
 from gsecrets.widgets.attribute_entry_row import AttributeEntryRow
 from gsecrets.widgets.entry_page_icon import EntryPageIcon
+from gsecrets.widgets.entry_page_tag import EntryPageTag
 from gsecrets.widgets.history_window import HistoryWindow
 from gsecrets.widgets.notes_dialog import NotesDialog
 from gsecrets.widgets.protected_attribute_entry_row import ProtectedAttributeEntryRow
 
+if typing.TYPE_CHECKING:
+    from gi.repository import Gio
+    from pykeepass.attachment import Attachment
+
+    from gsecrets.safe_element import SafeElement
+    from gsecrets.unlocked_database import UnlockedDatabase
+
 
 @Gtk.Template(resource_path="/org/gnome/World/Secrets/gtk/entry_page.ui")
 class EntryPage(Adw.Bin):
@@ -43,6 +52,9 @@ class EntryPage(Adw.Bin):
 
     icon_entry_box = Gtk.Template.Child()
 
+    _tags_group = Gtk.Template.Child()
+    _tag_entry_box = Gtk.Template.Child()
+
     attachments_preferences_group = Gtk.Template.Child()
     attachment_list_box = Gtk.Template.Child()
 
@@ -60,7 +72,14 @@ class EntryPage(Adw.Bin):
 
     safe_entry = GObject.Property(type=SafeEntry)
 
-    def __init__(self, u_d, entry, add_all):
+    _tags: list[EntryPageTag] = []
+
+    def __init__(
+        self,
+        u_d: UnlockedDatabase,
+        entry: SafeElement,
+        add_all: bool,
+    ) -> None:
         # Setup actions, must be done before initializing the template.
         self.install_action("entry.copy_password", None, self._on_copy_action)
         self.install_action("entry.copy_user", None, self._on_copy_action)
@@ -88,6 +107,7 @@ class EntryPage(Adw.Bin):
             self.attachments_preferences_group,
             self.attributes_preferences_group,
             self.expiration_date_preferences_group,
+            self._tags_group,
         ]
 
         self.insert_entry_properties_into_listbox(add_all)
@@ -120,7 +140,7 @@ class EntryPage(Adw.Bin):
 
         Gtk.Widget.do_unroot(self)
 
-    def insert_entry_properties_into_listbox(self, add_all):
+    def insert_entry_properties_into_listbox(self, add_all: bool) -> None:
         # pylint: disable=too-many-locals
         # pylint: disable=too-many-branches
         # pylint: disable=too-many-statements
@@ -193,6 +213,10 @@ class EntryPage(Adw.Bin):
             self.on_entry_icon_button_toggled,
         )
 
+        # Tags
+        self._update_tags()
+        self.show_row(self._tags_group, bool(safe_entry.tags), add_all)
+
         # Attachments
         for attachment in safe_entry.attachments:
             self.add_attachment_row(attachment)
@@ -229,6 +253,76 @@ class EntryPage(Adw.Bin):
                 self.show_all_preferences_group.props.visible = True
                 break
 
+    def _update_tags(self):
+        safe_entry = self.props.safe_entry
+
+        for tag in self._tags:
+            self._tag_entry_box.remove(tag)
+
+        self._tags = []
+
+        if safe_entry.tags:
+            for tag in safe_entry.tags:
+                widget = EntryPageTag(tag)
+                widget.remove_button.connect(
+                    "clicked",
+                    self._on_remove_button_clicked,
+                    tag,
+                )
+                self._tag_entry_box.append(widget)
+                self._tags.append(widget)
+
+    def _on_remove_button_clicked(self, _button, tag_name):
+        safe_entry = self.props.safe_entry
+        # Remove
+        tags = safe_entry.tags
+        tags.remove(tag_name)
+        safe_entry.tags = tags if tags else ""
+
+        # Update tag flowbox
+        self._update_tags()
+
+    @Gtk.Template.Callback()
+    def _on_tag_add_clicked(self, _widget):
+        entry = Adw.EntryRow()
+        entry.props.activates_default = True
+        entry.props.title = _("Enter tag name")
+
+        group = Adw.PreferencesGroup()
+        group.add(entry)
+
+        dialog = Adw.AlertDialog.new()
+        dialog.set_heading(_("Add New Tag"))
+        dialog.add_response("cancel", _("_Cancel"))
+        dialog.add_response("add", _("_Add"))
+        dialog.set_default_response("add")
+        dialog.set_close_response("cancel")
+        dialog.set_extra_child(group)
+
+        dialog.connect("response::add", self._on_tag_add_response, entry)
+        dialog.present(self.unlocked_database.window)
+
+        entry.grab_focus()
+
+    def _on_tag_add_response(
+        self,
+        _dialog: Adw.MessageDialog,
+        _response: str,
+        entry: Adw.EntryRow,
+    ) -> None:
+        tag_name = entry.props.text
+        safe_entry = self.props.safe_entry
+        tags = safe_entry.tags or []
+
+        if tag_name in tags:
+            window = self.unlocked_database.window
+            window.send_notification(_("The tag already exists"))
+            return
+
+        tags.append(tag_name)
+        safe_entry.tags = tags
+        self._update_tags()
+
     #
     # Events
     #
@@ -251,7 +345,12 @@ class EntryPage(Adw.Bin):
         else:
             self.action_set_enabled("entry.password_history", True)
 
-    def _on_copy_action(self, _widget, action_name, _parameter):
+    def _on_copy_action(
+        self,
+        _widget: Gtk.Widget,
+        action_name: str,
+        _parameter: GLib.Variant,
+    ) -> None:
         if self.unlocked_database.props.database_locked:
             return
 
@@ -321,7 +420,7 @@ class EntryPage(Adw.Bin):
         launcher.launch(window, None, self._on_launch, window)
 
     @Gtk.Template.Callback()
-    def on_otp_copy_button_clicked(self, _button):
+    def on_otp_copy_button_clicked(self, _button: Gtk.Button) -> None:
         safe_entry: SafeEntry = self.props.safe_entry
         otp_token = safe_entry.otp_token() or ""
         self.unlocked_database.send_to_clipboard(
@@ -360,7 +459,11 @@ class EntryPage(Adw.Bin):
             self._on_select_filechooser_response,
         )
 
-    def _on_select_filechooser_response(self, dialog, result):
+    def _on_select_filechooser_response(
+        self,
+        dialog: Gtk.FileDialog,
+        result: Gio.AsyncResult,
+    ) -> None:
         try:
             files = dialog.open_multiple_finish(result)
         except GLib.Error as err:
@@ -383,9 +486,11 @@ class EntryPage(Adw.Bin):
             for attachment in files:
                 attachment.load_bytes_async(None, callback)
 
-    def add_attachment_row(self, attachment):
+    def add_attachment_row(self, attachment: Attachment) -> None:
         safe_entry: SafeEntry = self.props.safe_entry
-        attachment_row = AttachmentEntryRow(safe_entry, attachment)
+        attachment_row = AttachmentEntryRow(
+            safe_entry, attachment, self.attachment_list_box
+        )
         self.attachment_list_box.append(attachment_row)
 
     @Gtk.Template.Callback()
