@@ -5,7 +5,7 @@ from __future__ import annotations
 import binascii
 import logging
 import typing
-from datetime import datetime, timezone
+from datetime import UTC, datetime
 from enum import Enum
 from gettext import gettext as _
 from typing import NamedTuple
@@ -56,6 +56,25 @@ class EntryColor(Enum):
             case EntryColor.BROWN:
                 return _("Brown")
 
+    def css_class(self):
+        match self:
+            case EntryColor.NONE:
+                return "white"
+            case EntryColor.BLUE:
+                return "blue"
+            case EntryColor.GREEN:
+                return "green"
+            case EntryColor.YELLOW:
+                return "yellow"
+            case EntryColor.ORANGE:
+                return "orange"
+            case EntryColor.RED:
+                return "red"
+            case EntryColor.PURPLE:
+                return "purple"
+            case EntryColor.BROWN:
+                return "brown"
+
 
 class SafeElement(GObject.Object):
     selected = GObject.Property(type=bool, default=False)
@@ -459,6 +478,9 @@ class SafeGroup(SafeElement):
         group = self.group
         force: bool = self._db_manager.check_entry_in_group_exists("", group)
 
+        if username == "":
+            username = self._db_manager.default_username
+
         new_entry = self._db_manager.db.add_entry(
             group,
             title,
@@ -620,7 +642,7 @@ class SafeEntry(SafeElement):
         """
         return self._entry
 
-    def duplicate(self):
+    def duplicate(self) -> None:
         """Duplicate an entry."""
         title: str = self.name or ""
         username: str = self.username or ""
@@ -669,9 +691,10 @@ class SafeEntry(SafeElement):
         if not self.props.expires:
             return
 
-        if self.props.expired:
-            self.notify("expired")
-        else:
+        # Update state
+        self.notify("expired")
+
+        if not self.props.expired:
             self._expired_id = GLib.timeout_add_seconds(600, self._is_expired)
 
     def _is_expired(self) -> bool:
@@ -765,14 +788,15 @@ class SafeEntry(SafeElement):
         if self.props.attributes.get(key) == value:
             return
 
-        self._attributes.insert(key, value)
-
         if self._entry.get_custom_property(
             key,
         ) == value and protected == self._entry.is_custom_property_protected(key):
             return
 
         self._entry.set_custom_property(key, value, protect=protected)
+
+        self._attributes.insert(key, value)
+
         self.updated()
         self.notify("attributes")
 
@@ -854,6 +878,15 @@ class SafeEntry(SafeElement):
         """
         return self.props.icon.name
 
+    @property
+    def tags(self) -> dict:
+        return self.entry.tags
+
+    @tags.setter  # type: ignore
+    def tags(self, new_tags: dict) -> None:
+        self.entry.tags = new_tags
+        self._db_manager.is_dirty = True
+
     @GObject.Property(type=str, default="")
     def otp(self) -> str:
         if self._otp:
@@ -987,6 +1020,7 @@ class SafeEntry(SafeElement):
             self.entry.expires = value
             self._check_expiration()
             self.updated()
+            self.notify("expired")
 
     @GObject.Property(
         type=bool,
@@ -1035,7 +1069,7 @@ class SafeEntry(SafeElement):
                 value.get_hour(),
                 value.get_minute(),
                 value.get_second(),
-                tzinfo=timezone.utc,
+                tzinfo=UTC,
             )
             self.entry.expiry_time = expired
             self._check_expiration()
@@ -1077,7 +1111,7 @@ ICONS = {
     "17": Icon("media-optical-cd-audio-symbolic"),
     "18": Icon("video-display-symbolic"),
     "19": Icon("mail-unread-symbolic", True),
-    "20": Icon("emblem-system-symbolic"),
+    "20": Icon("settings-symbolic"),
     "21": Icon("edit-paste-symbolic"),
     "22": Icon("edit-paste-symbolic"),
     "23": Icon("display-with-window-symbolic", True),
