@@ -5,12 +5,15 @@ import logging
 from enum import IntEnum
 from gettext import gettext as _
 
-from gi.repository import Adw, Gio, GLib, GObject, Gtk
+from gi.repository import Adw, Gio, GLib, Gtk
 
 import gsecrets.config_manager
+from gsecrets import const
 from gsecrets.create_database import CreateDatabase
 from gsecrets.database_manager import DatabaseManager
+from gsecrets.err import QUARK, ErrorType
 from gsecrets.provider.providers import Providers
+from gsecrets.recent_manager import RecentManager
 from gsecrets.save_dialog import SaveDialog
 from gsecrets.settings_dialog import SettingsDialog
 from gsecrets.unlock_database import UnlockDatabase
@@ -34,12 +37,10 @@ class Window(Adw.ApplicationWindow):
 
     _create_database_bin = Gtk.Template.Child()
     _main_view = Gtk.Template.Child()
-    _spinner = Gtk.Template.Child()
     toast_overlay = Gtk.Template.Child()
     _unlock_database_bin = Gtk.Template.Child()
     unlocked_db_bin = Gtk.Template.Child()
 
-    mobile_layout = GObject.Property(type=bool, default=False)
     _view = View.WELCOME
 
     def __init__(self, *args, **kwargs):
@@ -88,9 +89,6 @@ class Window(Adw.ApplicationWindow):
         ):
             create_view.close_banner()
 
-    def close_notification(self, toast: Adw.Toast) -> None:
-        toast.dismiss()
-
     def assemble_window(self) -> None:
         window_size = gsecrets.config_manager.get_window_size()
         self.set_default_size(window_size[0], window_size[1])
@@ -121,8 +119,9 @@ class Window(Adw.ApplicationWindow):
             # simply load the last opened file
             uri = gsecrets.config_manager.get_last_opened_database()
             gfile: Gio.File = Gio.File.new_for_uri(uri)
-            recents = Gtk.RecentManager.get_default()
-            if gfile.query_exists() and recents.has_item(uri):
+            recents = RecentManager()
+            recent_uris = [f.get_uri() for f in recents]
+            if gfile.query_exists() and uri in recent_uris:
                 filepath = gfile.get_path()
                 logging.debug("Opening last opened database: %s", filepath)
                 self.start_database_opening_routine(filepath)
@@ -288,7 +287,6 @@ class Window(Adw.ApplicationWindow):
 
     def create_new_database(self, filepath: str) -> None:
         """Copy stock database onto filepath."""
-        self._spinner.start()
         self._main_view.set_visible_child_name("spinner")
 
         stock_db_file: Gio.File = Gio.File.new_for_uri(
@@ -299,8 +297,10 @@ class Window(Adw.ApplicationWindow):
         def unlock_callback(database_manager, result):
             try:
                 database_manager.unlock_finish(result)
-            except GLib.Error:
-                logging.exception("Could not unlock safe")
+            except GLib.Error as err:
+                if not err.matches(QUARK, ErrorType.CREDENTIALS_ERROR):
+                    logging.exception("Could not unlock safe")
+
                 self.invoke_initial_screen()
                 self.send_notification(_("Could not create new Safe"))
             else:
@@ -308,8 +308,6 @@ class Window(Adw.ApplicationWindow):
                 self._create_database_bin.props.child = create_database
                 self._create_view = create_database
                 self.view = self.View.CREATE_DATABASE
-            finally:
-                self._spinner.stop()
 
         def copy_callback(gfile, result):
             try:
@@ -318,7 +316,6 @@ class Window(Adw.ApplicationWindow):
                 logging.exception("Could not copy new database")
                 self.invoke_initial_screen()
                 self.send_notification(_("Could not create new Safe"))
-                self._spinner.stop()
             else:
                 database_manager = DatabaseManager(self.key_providers, filepath)
                 database_manager.unlock_async(
@@ -543,14 +540,30 @@ class Window(Adw.ApplicationWindow):
 
     def on_about_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
         """Invoked when we click "about" in the main menu."""
-        builder = Gtk.Builder.new_from_resource(
-            "/org/gnome/World/Secrets/about_dialog.ui",
+        about_dialog = Adw.AboutDialog.new_from_appdata(
+            f"/org/gnome/World/Secrets/{const.APP_ID}.metainfo.xml",
         )
-        about_dialog = builder.get_object("about_dialog")
+        about_dialog.set_designers(["Christopher Davis", "Tobias Bernard"])
+        about_dialog.set_developers(
+            [
+                "Falk Alexander Seidl <fseidl@gnome.org>",
+                "Uta Lemke",
+                "David Heidelberg",
+                "Jean Felder",
+                "Maximiliano Sandoval <msandova@gnome.org>",
+                "Sebastian Spaeth",
+                "Christopher Davis",
+                "Jan-Michael Brummer",
+            ],
+        )
+        about_dialog.set_copyright("© 2018-2024 Falk Alexander Seidl, et al.")
+        # TRANSLATORS Add your name to the translator credits list
+        about_dialog.set_translator_credits(_("translator-credits"))
         about_dialog.present(self)
 
     def on_settings_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
-        SettingsDialog().present(self)
+        if self.get_visible_dialog() is None:
+            SettingsDialog().present(self)
 
     def on_go_back_action(self, _action: Gio.Action, _param: GLib.Variant) -> None:
         if self.view == self.View.CREATE_DATABASE:
